PROVIDER = "bedrock"

MODEL_TITAN_EXPRESS = "amazon.titan-text-express-v1"
EMBEDDING_MODEL_TITAN = "amazon.titan-embed-text-v1"

DEFAULT_MODEL = MODEL_TITAN_EXPRESS
DEFAULT_EMBEDDING_MODEL = EMBEDDING_MODEL_TITAN
