
PROVIDER = "ollama"

EMBEDDING_MODEL_NOMIC = "nomic-embed-text"
EMBEDDING_MODEL_NOMIC_TOKENIZER = "nomic-ai/nomic-embed-text-v1.5"
EMBEDDING_MODEL_MINILM = "all-minilm"
EMBEDDING_MODEL_MINILM_TOKENIZER = "sentence-transformers/all-MiniLM-L6-v2"

MODEL_PHI3 = "phi3"
MODEL_LLAMA3 = "llama3"

DEFAULT_MODEL = MODEL_PHI3
DEFAULT_EMBEDDING_MODEL = EMBEDDING_MODEL_NOMIC
